import pygame, sys

class Text:
    def __init__(self, window, text: str, fontFile = None, fontSize: int = 20, x = 0, y = 0, color = "white") -> None:
        self.window = window
        self.text = text
        self.fontFile = fontFile
        self.fontSize = fontSize
        self.x = x
        self.y = y
        self.color = color

        if sys.platform == "linux" or sys.platform == "linux2" or sys.platform == "win32" or sys.platform == "cygwin":
            self.font = pygame.font.SysFont(self.fontFile, self.fontSize)
        else:
            self.font = pygame.font.Font(self.fontFile, self.fontSize)

        self.text = self.font.render(self.text, True, self.color)

    def show(self):
        self.window.window.blit(self.text, (self.x, self.y))
        


import pygame

class Input:
    def __init__(self):
        self.pressed = pygame.key.get_pressed()
        self.mouse = pygame.mouse.get_pressed()

        self.keys = {
            "UP":pygame.K_UP, "DOWN":pygame.K_DOWN,
            "LEFT":pygame.K_LEFT, "RIGHT":pygame.K_RIGHT,

            "A":pygame.K_a, "B":pygame.K_b, "C":pygame.K_c,
            "D":pygame.K_d, "E":pygame.K_e, "F":pygame.K_f,
            "G":pygame.K_g, "H":pygame.K_h, "I":pygame.K_i,
            "J":pygame.K_j, "K":pygame.K_k, "L":pygame.K_l,
            "M":pygame.K_m, "N":pygame.K_n, "O":pygame.K_o,
            "P":pygame.K_p, "Q":pygame.K_q, "R":pygame.K_r,
            "S":pygame.K_s, "T":pygame.K_t, "U":pygame.K_u,
            "V":pygame.K_v, "W":pygame.K_w, "X":pygame.K_x,
            "Y":pygame.K_y, "Z":pygame.K_z,

            "0":pygame.K_0, "1":pygame.K_1,"2":pygame.K_2, 
            "3":pygame.K_3, "4":pygame.K_4, "5":pygame.K_5,
            "6":pygame.K_6, "7":pygame.K_7, "8":pygame.K_8,
            "9":pygame.K_9,

            "F1":pygame.K_F1, "F2":pygame.K_F2, "F3":pygame.K_F3,
            "F4":pygame.K_F4, "F5":pygame.K_F5, "F6":pygame.K_F6,
            "F7":pygame.K_F7, "F8":pygame.K_F8, "F9":pygame.K_F9,
            "F10":pygame.K_F10, "F11":pygame.K_F11, "F12":pygame.K_F12,
            "F13":pygame.K_F13, "F14":pygame.K_F14, "F15":pygame.K_F15,

            "SCAN_F1":pygame.KSCAN_F1, "SCAN_F2":pygame.KSCAN_F2, 
            "SCAN_F3":pygame.KSCAN_F3, "SCAN_F4":pygame.KSCAN_F4, 
            "SCAN_F5":pygame.KSCAN_F5, "SCAN_F6":pygame.KSCAN_F6,
            "SCAN_F7":pygame.KSCAN_F7, "SCAN_F8":pygame.KSCAN_F8, 
            "SCAN_F9":pygame.KSCAN_F9, "SCAN_F10":pygame.KSCAN_F10, 
            "SCAN_F11":pygame.KSCAN_F11, "SCAN_F12":pygame.KSCAN_F12,
            "SCAN_F13":pygame.KSCAN_F13, "SCAN_F14":pygame.KSCAN_F14, 
            "SCAN_F15":pygame.KSCAN_F15,

            "LCTRL":pygame.K_LCTRL, "RCTRL":pygame.K_RCTRL,
            "LSHIFT":pygame.K_LSHIFT, "RSHIFT":pygame.K_RSHIFT,
            "LALT":pygame.K_LALT, "RALT":pygame.K_RALT,
            "LSUPER":pygame.K_LSUPER, "RSUPER":pygame.K_RSUPER,

            "MOD_CTRL":pygame.KMOD_CTRL, "MOD_ALT":pygame.KMOD_ALT,
            "MOD_CAPS":pygame.KMOD_CAPS, "MOD_GUI":pygame.KMOD_GUI,
            "MOD_LALT":pygame.KMOD_LALT, "MOD_LCTRL":pygame.KMOD_LCTRL,
            "MOD_LGUI":pygame.KMOD_LGUI, "MOD_LMETA":pygame.KMOD_LMETA,
            "MOD_LSHIFT":pygame.KMOD_LSHIFT, "MOD_META":pygame.KMOD_META,
            "MOD_NONE":pygame.KMOD_NONE, "MOD_MODE":pygame.KMOD_MODE,
            "MOD_NUM":pygame.KMOD_NUM, "MOD_RALT":pygame.KMOD_RALT,
            "MOD_RSHIFT":pygame.KMOD_RSHIFT, "MOD_RCTRL":pygame.KMOD_RCTRL,
            "MOD_RMETA":pygame.KMOD_RMETA, "MOD_RGUI":pygame.KMOD_RGUI,
            "MOD_SHIFT":pygame.KMOD_SHIFT, "MODE":pygame.K_MODE,
            "SCAN_MODE":pygame.KSCAN_MODE,

            "HOME":pygame.K_HOME, "INSERT":pygame.K_INSERT,
            "DELETE":pygame.K_DELETE, "RETURN":pygame.K_RETURN,
            "SPACE":pygame.K_SPACE, "PLUS":pygame.K_PLUS,
        }

        self.mouse_buttons = {
            "LEFT":0, "MIDDLE":1, "RIGHT":2, 
        }

    def update(self):
        self.pressed = pygame.key.get_pressed()
        self.mouse = pygame.mouse.get_pressed()

    def get_key_pressed(self, key):
        return self.pressed[key]

    def get_mouse_button(self, button):
        return self.mouse[button]

    def get_mouse_pos(self):
        return pygame.mouse.get_pos()


import pygame

class Sprite:
    def __init__(self, window, picture: str, x = 0, y = 0, width: int = 200, height: int = 200):
        self.window = window
        self.picture = picture
        self.x = x
        self.y = y
        self.width = width
        self.height = height

        self.image = pygame.image.load(self.picture).convert()
        self.rect = pygame.Rect(0, 0, 0, 0)

        if width != None and height != None:
            self.image =  pygame.transform.scale(self.image, (width, height))

    def show(self):
        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)
        self.window.window.blit(self.image, (self.x, self.y))

    def rotate(self, angle):
        self.image = pygame.transform.scale(self.image, (self.width, self.height))
        self.image = pygame.transform.rotate(self.image, angle)

    def getWidth(self):
        return self.image.get_width()

    def getHeight(self):
        return self.image.get_height()

    def checkCollision(self, second_entity):
        try:
            if type(second_entity) != list:
                return self.rect.colliderect(second_entity.rect)
            elif type(second_entity) == list:
                return self.rect.collidelistall(second_entity.rect)
        except:
            raise AttributeError("Gamwo.Sprite.checkCollision: second_entity must be a Gamwo.Entity or a list of Gamwo.Entity or Gamwo.Sprite or a list of Gamwo.Sprite")


import pygame, sys

class Line:
    def __init__(self, window, start, end, color) -> None:
        self.window = window
        self.start = start
        self.end = end
        self.color = color

        pygame.draw.aaline(self.window.window, self.color, self.start, self.end)


import pygame, sys

class Entity(pygame.Rect):
    def __init__(self, window, shape = "square", width = 20, height = 20, x = 0, y = 0, color = "white"):
        self.window = window
        self.shape = shape
        self.width = width
        self.height = height
        self.x = x
        self.y = y
        self.color = color
        self.rect = self
        
    def show(self):
        if self.shape == "square" or self.shape == "rect" or self.shape == "rectangle":
            pygame.draw.rect(self.window.window, self.color, self)
        elif self.shape == "ellipse" or self.shape == "circle":
            pygame.draw.ellipse(self.window.window, self.color, self)

    def checkCollision(self, second_entity):
        try:
            if type(second_entity) != list:
                return self.colliderect(second_entity.rect)
            elif type(second_entity) == list:
                return self.collidelistall(second_entity.rect)
        except:
            raise AttributeError("Gamwo.Entity.checkCollision: second_entity must be a Gamwo.Entity or a list of Gamwo.Entity or Gamwo.Sprite or a list of Gamwo.Sprite or a list of Gamwo.Sprite") 


import threading

class Timer:
    def __init__(self, duration) -> None:
        self.duration = duration
        self.action_handler = []

    def _setAction(self):
        for action in self.action_handler:
            action()

    def start(self):
        threading.Timer(self.duration, self._setAction).start()

    def setStartAction(self, action):
        self.action_handler.append(action)
        return action


import pygame, sys
from gamwo.input import Input
from pygame.locals import *
import time

class Gamwo:
    '''Main class, this create a window, draw, update, detect input and more, its very important.'''

    def __init__(self, title: str = "Gamwo 2D!", width: int = 800, height: int = 600, target_fps: int = 60):

        '''To main class we have to give
        :param title: Window title
        :type title: str
        :param width: Window width
        :type width: int
        :param height: Window height
        :type height: int
        :return: None
        :rtype: None'''
        self.title = title
        self.width = width
        self.height = height
        self.target_fps = target_fps
        
        pygame.init()

        self.window = pygame.display.set_mode((self.width, self.height), pygame.DOUBLEBUF)
        pygame.display.set_caption(self.title)
        pygame.event.set_allowed([QUIT])
        self.window.set_alpha(None)

        self.clock = pygame.time.Clock()

        self.dt = 0
        self.prev_time = time.time()

        self.input = Input()

        self.current_time = 0

        self.update_deltatime = 0
        self.draw_handlers = []
        self.update_handlers = []

    def _draw(self):
        for handler in self.draw_handlers:
            handler()

    def _update(self, deltatime):
        for handler in self.update_handlers:
            handler(deltatime)

    def changeBackgroundColor(self, color):
        self.window.fill(color)

    def getFPS(self):
        return self.clock.get_fps()

    def run(self):
        while True:
            self.clock.tick(self.target_fps)

            self.now = time.time()
            self.dt = self.now - self.prev_time
            self.prev_time = self.now

            for e in pygame.event.get():
                if e.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()

            self.input.update()

            self.current_time = pygame.time.get_ticks()

            self._update(self.dt)
            self.changeBackgroundColor("black")
            self._draw()

            pygame.display.update()
            pygame.display.flip()

    def draw(self, handler):
        self.draw_handlers.append(handler)
        return handler

    def update(self, handler):
        self.update_handlers.append(handler)
        return handler

        


from setuptools import setup

setup(
    name = "Gamwo",
    packages = ['gamwo'],
    version = '0.1.1',
    description = 'Simple python framework for 2d games',
    long_description= 'There isnt much to say about this package',
    long_description_content_type='text/markdown',
    author='RinkaDev',
    author_email='rinkadevoficial@gmail.com',
    url='https://github.com/RinkaGI/Gamwo',
    keywords=['game', 'framework', '2d', 'simple', 'pygame'],
    classifiers=[ ],
    license='MIT',
    include_package_data=True
)